@@ -1,14 +1,60 @@
+import datetime
 import os
 from enum import unique, Enum
+from typing import IO
 
 import requests
 import yaml
 from requests_toolbelt import MultipartEncoder
 
+import cjen
 from cjen.mama.operate.json import factory
 from cjen.bigtangerine import BigTangerine
 from cjen.commons import _get_method_params
 from cjen.exceptions import _check_uri, NetWorkErr, _check_instance
+import cjen.dada.smile
+
+
+def url_log_content(resp, log_type):
+    date = datetime.datetime.now().strftime('%y-%m-%d %I:%M:%S')
+    method = resp.request.method
+    url = resp.url
+    return "{date} {log_type} {method} {url}\n".format(date=date, log_type=log_type, method=method, url=url)
+
+
+def body_log_content(resp, io: IO):
+    request_body_content(resp.request.body, io)
+    response_body_content(resp.content, io)
+
+
+def request_body_content(request_body, io: IO):
+    if request_body and type(request_body) == bytes:
+        io.write("  -- request {request_body}\n".format(request_body=str(request_body, encoding="UTF-8")))
+    if request_body and type(request_body) == MultipartEncoder:
+        io.write("  -- request {request_body}\n".format(request_body=str(request_body)))
+
+
+def response_body_content(resp_body, io: IO):
+    if resp_body:
+        io.write("  -- response {resp_body}\n".format(resp_body=str(resp_body, encoding="UTF-8")))
+
+
+@cjen.haha(LogPath=os.getcwd(), LogName="httpd.log", Mode='a')
+def httpd_log(resp, io: IO):
+    io.write(url_log_content(resp, "info"))
+    body_log_content(resp, io)
+
+
+@cjen.haha(LogPath=os.getcwd(), LogName="httpd.log", Mode='a')
+def httpd_err_log(resp, io: IO):
+    io.write(url_log_content(resp, "error"))
+    body_log_content(resp, io)
+
+
+@cjen.haha(LogPath=os.getcwd(), LogName="httpd.log", Mode='a')
+def httpd_connection_err_log(err, io: IO):
+    date = datetime.datetime.now().strftime('%y-%m-%d %I:%M:%S')
+    io.write("{date} error {err}\n".format(date=date, err=str(err)))
 
 
 def _multipart_form(func):
@@ -50,15 +96,18 @@ class ContentType(Enum):
 
 def _response(func):
     def __inner__(ins: BigTangerine, *args, **kwargs):
-
         if kwargs.get("resp").status_code == 200:
+            httpd_log(kwargs.get("resp"))
             if "application/json" in kwargs.get("resp").headers.get("Content-Type"):
                 kwargs["response_content"] = kwargs.get("resp")
+
                 kwargs["resp"] = kwargs.get("resp").json() if kwargs.get("resp").content else None
+
                 return func(ins, *args, **kwargs)
             kwargs["resp"] = kwargs.get("resp").content
             return func(ins, *args, **kwargs)
         else:
+            httpd_err_log(kwargs.get("resp"))
             raise NetWorkErr(f'{kwargs.get("url")} {kwargs.get("resp").status_code}')
 
     return __inner__
@@ -99,6 +148,13 @@ def _delete(func):
     return __inner__
 
 
+# def _get(func):
+#     def __inner__(ins: BigTangerine, *args, **kwargs):
+#         headers = {**ins.headers, **kwargs.get("headers")} if kwargs.get("headers") else ins.headers
+#         kwargs["resp"] = requests.get(url=kwargs["url"], headers=headers,
+#                                       params=kwargs.get("params"))
+
+
 def _http_headers(func):
     def __inner__(ins: BigTangerine, *args, **kwargs):
         kwargs["headers"] = {**ins.headers, **kwargs.get("headers")} if kwargs.get("headers") else ins.headers
@@ -123,7 +179,10 @@ def _http_json(*, method):
     def __wrapper__(func):
         def __inner__(ins: BigTangerine, *args, **kwargs):
             if _resp_is_not_none(**kwargs) and _have_content_type(**kwargs) and _is_json(**kwargs):
-                kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], json=kwargs.get("data"))
+                try:
+                    kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], json=kwargs.get("data"))
+                except OSError as err:
+                    httpd_connection_err_log(err)
             return func(ins, *args, **kwargs)
 
         return __inner__
@@ -135,7 +194,10 @@ def _http_file(*, method):
     def __wrapper__(func):
         def __inner__(ins: BigTangerine, *args, **kwargs):
             if _resp_is_not_none(**kwargs) and _have_content_type(**kwargs) and _is_upload(**kwargs):
-                kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], data=kwargs.get("form_data"))
+                try:
+                    kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], data=kwargs.get("form_data"))
+                except OSError as err:
+                    httpd_connection_err_log(err)
             return func(ins, *args, **kwargs)
 
         return __inner__
@@ -147,7 +209,10 @@ def _http_default(*, method):
     def __wrapper__(func):
         def __inner__(ins: BigTangerine, *args, **kwargs):
             if _resp_is_not_none(**kwargs):
-                kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], data=kwargs.get("data"))
+                try:
+                    kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], data=kwargs.get("data"))
+                except OSError as err:
+                    httpd_connection_err_log(err)
             return func(ins, *args, **kwargs)
 
         return __inner__
@@ -159,6 +224,7 @@ def _get(func):
     def __inner__(ins: BigTangerine, *args, **kwargs):
         headers = {**ins.headers, **kwargs.get("headers")} if kwargs.get("headers") else ins.headers
         kwargs["resp"] = requests.get(url=kwargs["url"], headers=headers, params=kwargs.get("params"))
+
         return func(ins, *args, **kwargs)
 
     return __inner__
@@ -224,6 +290,7 @@ def post_mapping(*, uri: str, json_clazz=None):
     发送 Post 请求，并返回结果
 
     :param json_clazz:
+
     :param uri:
     :return:
     """
@@ -254,6 +321,7 @@ def put_mapping(*, uri: str, json_clazz=None):
     发送 PUT 请求，并返回结果
 
     :param json_clazz:
+
     :param uri:
     :return:
     """
